import io, os, sys, time, pygame, random
import photoboite.screens, photoboite.buttons, photoboite.buttons.config

try:
    import picamera

    class Camera:
        def __init__(self, rect):
            self.rect = rect
            self.camera = picamera.PiCamera()
            self.camera.led = False
            self.camera.resolution = (self.rect.width, self.rect.height)
            self.rgb = bytearray(self.rect.width * self.rect.height * 4)

        def close(self):
            self.camera.close()

        def take(self, name):
            name = name + '.jpg'
            self.camera.resolution = (3280, 2464)
            self.camera.capture(name)
            self.camera.resolution = (self.rect.width, self.rect.height)
            return name

        def stream(self):
            stream = io.BytesIO()
            self.camera.capture(stream, use_video_port=True, format='rgba', resize=(self.rect.width, self.rect.height))
            stream.seek(0)
            stream.readinto(self.rgb)
            stream.close()
            img = pygame.image.frombuffer(self.rgb, (self.rect.width, self.rect.height), 'RGBA')

            return pygame.transform.flip(img, True, False)

        def draw_on(self, screen):
            screen.blit(self.stream(), self.rect)


except:
    class Camera:
        def __init__(self, rect):
            self.rect = rect
            self.img = pygame.transform.scale(pygame.image.load(os.path.dirname(os.path.realpath(__file__)) + '/../images/tournesol.jpg'), (self.rect.width, self.rect.height))

        def close(self):
            pass

        def take(self, name):
            print('Take picture! ' + name)
            return os.path.dirname(os.path.realpath(__file__)) + '/../images/tournesol.jpg'

        def draw_on(self, screen):
            screen.blit(self.img, self.rect)



class Capture:
    def __init__(self, uid):
        self.uid = uid
        self.photos = []

    def take(self, camera):
        self.photos.append(Photo(self.uid, len(self.photos)).take(camera))


class Photo:
    def __init__(self, cid, uid):
        self.cid = cid
        self.uid = uid

    def take(self, camera):
        self.name = "photos/%s-%04d-%04d" % (time.strftime('%Y%m%d-%H%M%S'), self.cid, self.uid)

        surface = pygame.Surface((646, 488))
        surface.fill((255, 255, 255))
        surface.blit(pygame.transform.smoothscale(pygame.image.load(camera.take(self.name)), (638, 480)), (4, 4))

        self.capture = pygame.transform.rotate(surface.convert_alpha(), random.randrange(-20, 20, 4))

        return self


class Photoboite:
    def __init__(self):
        self.screen = self.screen()
        pygame.font.init()

        pygame.mouse.set_visible(False)

        self.clock = pygame.time.Clock()

        self.config = photoboite.screens.Config(self)
        self.sleep = photoboite.screens.Sleep(30, self)
        self.capture = photoboite.screens.Capture(self, self.sleep)


    def screen(self):
        if os.getenv('DISPLAY'):
            pygame.display.init()
            display = pygame.display.Info()
            return pygame.display.set_mode((800, 480))
        else:
            drivers = ['directfb', 'fbcon', 'svgalib']
            found = False
            for driver in drivers:
                if not os.getenv('SDL_VIDEODRIVER'):
                    os.putenv('SDL_VIDEODRIVER', driver)
                try:
                    pygame.display.init()
                except pygame.error as e:
                    print('Driver: {0} failed.'.format(driver))
                    continue
                found = True
                break

            if not found:
               raise Exception('No suitable video driver found!')

            return pygame.display.set_mode()


    def run(self):
        while True:
            if self.config.enabled:
                self.config.run()
            elif self.sleep.enabled:
                self.sleep.run()
            else:
                self.capture.run()

            self.clock.tick(60)

    def quit(self):
        self.capture.quit()
        sys.exit()
